/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * drawable.h
 * ----------
 *
 * A box that can be drawn on
 * Includes all ROWS graphics functions in combination with ROWSbrush
 */

#ifndef __impulse_rows_drawable_h
#define __impulse_rows_drawable_h

#include <impulse/rows/box.h>

/*                            ======--------======                           */

/* Drawable widget type */

typedef void **ROWS_drawable;

/*                            ======--------======                           */

/* 
 * These include files are after the definition of ROWS_drawable, cos that
 * is needed by ROWS_bitmap & ROWS_style
 */

#include <impulse/rows/bitmap.h>
#include <impulse/rows/style.h>

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget     ROWS_drawable_create         (void);
extern ROWS_widget    *ROWS_drawable_initialise     (int id);
extern void            ROWS_drawable_postinitialise (ROWS_widget drawable,
						     int         has_styles);
extern inline int      ROWS_drawable_next_identifier(void);
extern inline ROWS_drawable ROWS_drawable_cast           (void **widget);

/*                            ======--------======                           */

/* Class-specific functions */

/* Drawable validity functions */

extern void ROWS_drawable_set_valid(ROWS_drawable drawable,
				    int           valid);
extern int  ROWS_drawable_get_valid(ROWS_drawable drawable);

/* Clipping stuff */


extern void ROWS_drawable_clip_rect(ROWS_drawable  drawable,
				    ROWS_rect     *clip);
extern void ROWS_drawable_clip_push(ROWS_drawable  drawable);
extern void ROWS_drawable_clip_pop(ROWS_drawable  drawable);
extern void ROWS_drawable_clip_clear(ROWS_drawable  drawable);
extern void ROWS_drawable_clip_union(ROWS_drawable  drawable,
				     ROWS_rect     *clip);
extern void ROWS_drawable_clip_intersect(ROWS_drawable  drawable,
					 ROWS_rect     *clip);
extern void ROWS_drawable_clip_subtract(ROWS_drawable  drawable,
					ROWS_rect     *clip);
extern void ROWS_drawable_clip_set(ROWS_drawable  drawable);
extern void ROWS_drawable_clip_bounds(ROWS_drawable  drawable,
				      ROWS_rect     *rect);

extern void ROWS_drawable_clip_move(ROWS_drawable  drawable,
				    ROWS_point    *offset);
extern void ROWS_drawable_clip_copy(ROWS_drawable  to,
				    ROWS_drawable  from,
				    ROWS_point    *offset);

extern int  ROWS_drawable_get_clip_serial(ROWS_drawable drawable);

/* Style stuff */

extern void        ROWS_drawable_set_style(ROWS_drawable drawable,
					   ROWS_style style);
extern ROWS_style ROWS_drawable_get_style(ROWS_drawable drawable);

/* Drawing functions */

extern void ROWS_drawable_clear_area      (ROWS_drawable drawable,
					   int x, int y,
					   int w, int h);
extern void ROWS_drawable_draw_bitmap     (ROWS_drawable drawable,
					   ROWS_bitmap   bitmap,
					   microns       x,
					   microns       y);
extern void ROWS_drawable_sync            (ROWS_drawable drawable);

extern ROWS_gamut ROWS_drawable_get_gamut(ROWS_drawable drawable);

#endif
