/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * drawable_int.h
 * --------------
 *
 * Drawable internals
 */

#ifndef __impulse_rows_drawable_int_h
#define __impulse_rows_drawable_int_h

#include <impulse/rows/drawable.h>


/*
 * Drawable function types
 */

typedef enum {
  ROWS_DRAWABLE_COLOUR_GET_XYZ = 0,
  ROWS_DRAWABLE_COLOUR_GET_GAMUT,
  ROWS_DRAWABLE_COLOUR_CREATE,
  ROWS_DRAWABLE_COLOUR_RESET,
  ROWS_DRAWABLE_COLOUR_NAMED,
  ROWS_DRAWABLE_COLOUR_PIXEL,
  ROWS_DRAWABLE_COLOUR_DISCARD,

  ROWS_DRAWABLE_BRUSH_CREATE,
  ROWS_DRAWABLE_BRUSH_DESTROY,

  ROWS_DRAWABLE_BRUSH_SET_GC,
  ROWS_DRAWABLE_BRUSH_USE_PRIVATE_GC,

  ROWS_DRAWABLE_BRUSH_SET_LINEWIDTH,
  ROWS_DRAWABLE_BRUSH_SET_FGCOLOUR,
  ROWS_DRAWABLE_BRUSH_SET_BGCOLOUR,
  ROWS_DRAWABLE_BRUSH_SET_DOTPATTERN,
  ROWS_DRAWABLE_BRUSH_GET_FGPIXEL,

  ROWS_DRAWABLE_BRUSH_DRAW_POINT,
  ROWS_DRAWABLE_BRUSH_DRAW_LINE,
  ROWS_DRAWABLE_BRUSH_DRAW_ARC,
  ROWS_DRAWABLE_BRUSH_FILL_ARC,
  ROWS_DRAWABLE_BRUSH_DRAW_RECTANGLE,
  ROWS_DRAWABLE_BRUSH_FILL_RECTANGLE,
  ROWS_DRAWABLE_BRUSH_DRAW_SHAPE,
  ROWS_DRAWABLE_BRUSH_FILL_SHAPE,
  
  ROWS_DRAWABLE_BRUSH_SET_FONT,
  ROWS_DRAWABLE_BRUSH_SET_FONT_SIZE,
  ROWS_DRAWABLE_BRUSH_FONT_PAINT,
  ROWS_DRAWABLE_BRUSH_FONT_PAINT_MATRIX,
  ROWS_DRAWABLE_BRUSH_FONT_MEASURE,
  ROWS_DRAWABLE_BRUSH_FONT_MEASURE_SPLIT,
  ROWS_DRAWABLE_BRUSH_GET_FONT_LIST,

  ROWS_DRAWABLE_CLEAR_AREA,
  ROWS_DRAWABLE_DRAW_BITMAP,
  ROWS_DRAWABLE_SYNC,

  ROWS_DRAWABLE_CLIP_PUSH,
  ROWS_DRAWABLE_CLIP_POP,
  ROWS_DRAWABLE_CLIP_CLEAR,
  ROWS_DRAWABLE_CLIP_UNION,
  ROWS_DRAWABLE_CLIP_INTERSECT,
  ROWS_DRAWABLE_CLIP_SUBTRACT,
  ROWS_DRAWABLE_CLIP_SET,
  ROWS_DRAWABLE_CLIP_BOUNDS,
  ROWS_DRAWABLE_CLIP_RECT,
  ROWS_DRAWABLE_CLIP_MOVE,
  ROWS_DRAWABLE_CLIP_COPY,

  ROWS_DRAWABLE_MAX_FUNC
} ROWS_drawable_function_type;

/*
 * Drawable function parameters (describes parameters for all possible 
 * functions)
 */

typedef union _ROWS_drawable_fparam {
  struct {
    ROWS_drawable drawable;

    union {
      struct {
	ROWS_colour_description colour;

	int x,y,z; /* (output) */
      } get_xyz;
      struct {
	ROWS_colour_desc xyz;
	ROWS_colour_desc rgb;

	impulse_bool     is_spot;
	
	void *data;
      } create; /* or Reset... */
      struct {
	ROWS_gamut gamut;
      } get_gamut;
      struct {
      } get_screen_info;
      struct {
	impulse_int pixel;

	void *data; /* return value */
      } create_pixel;
      struct {
	void *data;
      } discard;
    } data;
  } colour;
  
  struct {
    ROWS_brush brush;
    
    union {
      struct {
	impulse_int value;
      } singleint; /* set_linewidth, get_fgpixel (return value in the
		    * int) */
      struct {
	microns width;
      } linewidth;
      struct {
	microns offset;
	ROWS_brush_dotpattern dp;
      } dotpattern; /* set_dotpattern */
      struct {
	ROWS_colour colour;
      } colour;
      struct {
	microns x, y;
      } coordinate; /* draw_point */
      struct {
	microns x1, y1, x2, y2;
      } line; /* draw_line, draw_rectangle */
      struct {
	microns x, y, w, h;
	impulse_short angle1, angle2;
      } arc; /* draw_arc */
      struct {
	ROWS_brush_shape *shape;
	int nopoints;
      } shape; /* draw_shape, fill_shape */

      struct {
	impulse_char *font_name;
      } set_font;
      struct {
	microns sizex, sizey;
      } set_font_size;
      struct {
	impulse_int use_matrix;
	impulse_unistring  text;
	microns            x,y;
	impulse_float     *matrix;
      } font_paint; /* font_paint, font_paint_matrix */
      struct {
	impulse_unistring   text;
	ROWS_brush_measure *result;
      } font_measure;
      struct {
	impulse_unistring text;
	microns           maxwidth;
	impulse_int result;
      } font_measure_split;
      struct {
	ROWS_brush_fontlist result;
      } get_font_list;
    } data;
    
  } brush; /* set_gc, use_private_gc */
  
  struct {
    ROWS_drawable drawable;
    
    union {
      struct {
	microns x, y, w, h;
      } clear_area;
      struct {
	ROWS_rect rect;
      } clip; /* set_clip, clip_region_add, set, bounds */
      struct {
	ROWS_drawable from;
	ROWS_point offset;
      } copy; /* clip_region_copy, clip_region_move  */
      struct {
	ROWS_bitmap bitmap;
	microns x;
	microns y;
      } draw_bitmap;
    } data;
  } drawable;
} ROWS_drawable_fparam;

/* Abstract drawable function */

typedef void (*ROWS_drawable_abstract_func)(ROWS_drawable_fparam *param);

/* 
 * Call routines for a brush (not as nice as abstract functions, but at
 * least you get a good idea of what's going on)
 */

struct _ROWS_drawable_brush_calls {
  ROWS_drawable_abstract_func function[ROWS_DRAWABLE_MAX_FUNC];
};

typedef struct _ROWS_drawable_brush_calls ROWS_drawable_brush_calls;

/* Drawable data structure */

typedef struct {
  ROWS_style style;
  impulse_int valid;

  ROWS_brush last_brush;

  impulse_int clip_serial;

  ROWS_drawable_brush_calls abstraction_layer;
} _p_ROWS_drawable_data;

/*                            ======--------======                           */

/* Font abstraction layer */

typedef void (*ROWS_drawable_set_font)(ROWS_drawable drawable,
				       ROWS_brush  brush,
				       char       *font,
				       char       *architechture,
				       void       *data);
typedef void (*ROWS_drawable_unset_font)(ROWS_drawable drawable,
					 ROWS_brush  brush,
					 char       *font,
					 char       *architechture,
					 void       *data);
typedef void (*ROWS_drawable_font_paint)(ROWS_drawable drawable,
					 ROWS_brush  brush,
					 char       *font,
					 microns     x,
					 microns     y,
					 char       *arch,
					 void       *data);
typedef void (*ROWS_drawable_font_paint_matrix) (ROWS_drawable       drawable,
						 ROWS_brush          brush,
						 impulse_unistring   text,
						 microns             x,
						 microns             y,
						 float              *matrix,
						 char               *arch,
						 void               *data);
typedef void (*ROWS_drawable_font_measure)(ROWS_drawable       drawable,
					   ROWS_brush          brush,
					   impulse_unistring   text,
					   ROWS_brush_measure *result,
					   char               *arch,
					   void               *data);
typedef int (*ROWS_drawable_font_measure_split)(ROWS_drawable       drawable,
						ROWS_brush          brush,
						impulse_unistring   text,
						microns             maxwidth,
						char               *arch,
						void               *data);
typedef int (*ROWS_drawable_font_measure_wrap) (ROWS_drawable       drawable,
						ROWS_brush          brush,
						impulse_unistring   text,
						microns             maxwidth,
						impulse_unichar     brk,
						char               *arch,
						void               *data);

typedef struct _ROWS_drawable_font_renderer {
  char *renderer_name;

  ROWS_drawable_set_font           set_font;
  ROWS_drawable_unset_font         unset_font;
  ROWS_drawable_font_paint         font_paint;
  ROWS_drawable_font_paint_matrix  font_paint_matrix;
  ROWS_drawable_font_measure       font_measure;
  ROWS_drawable_font_measure_split font_measure_split;
  ROWS_drawable_font_measure_wrap  font_measure_wrap;

  void *private_data;

  int features;
} ROWS_drawable_font_renderer;

/* Font renderer features */
#define ROWS_DRAWABLE_FONT_CAN_MATRIX  1
#define ROWS_DRAWABLE_FONT_IS_SCALABLE 2

/*                            ======--------======                           */

/* Abstraction functions */

extern ROWS_drawable_brush_calls *ROWS_drawable_get_abstraction_layer(ROWS_drawable drawable);
extern void ROWS_drawable_register_font_renderer(char *graphics,
						 char *architechture,
						 ROWS_drawable_font_renderer renderer,
						 void *data);
						 

#endif
