/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * memory.c
 * --------
 *
 * Memory handling functions
 */

#ifndef __impulse_util_memory_h
#define __impulse_util_memory_h

#include <stddef.h>

#ifdef __cplusplus
extern "C" {  
#endif

  void impulse_mcheck   (void);
  void impulse_mem_used (void *mem);
  int  impulse_mem_count(void *mem);
  
#ifndef CHECK_MEMORY_LEAKS
  void *impulse_alloc(size_t size);
  void *impulse_realloc(void *old, size_t size);
  void impulse_free(void *old);
#else
  void *impulse_alloc_c(size_t size,
			int line_no,
			char *file,
			char *comment);
  void *impulse_realloc(void   *old,
			size_t  size);
  void  impulse_free(void *old);

#define impulse_alloc(size) impulse_alloc_c(size, __LINE__, __FILE__, NULL)
#define impulse_alloc_str(size, comment) impulse_alloc_c(size, __LINE__, __FILE__, comment)
#endif

#ifdef __cplusplus
}
#endif

#endif

/* End of file */
